#include "main.h"

static Evas_Object *o_bg_rect  = NULL;
static Evas_Object *o_bg       = NULL;
static Evas_Object *o_bg_init  = NULL;

static int  _bg_cb_timer_init_end(void *data);
static void _bg_cb_key_down(void *data, Evas *e, Evas_Object *obj, void *event_info);
static void _bg_cb_stop_done(void *data, Evas_Object *o, const char *emission, const char *source);

void
bg_resize(double w, double h)
{
   evas_object_resize(o_bg_rect, w, h);
}

void
bg_start(void)
{
   Evas_Object *o;
   Evas_Coord w, h;

   evas_output_viewport_get(evas, NULL, NULL, &w, &h);
   
   o = evas_object_rectangle_add(evas);
   evas_object_color_set(o, 255, 255, 255, 255);
   evas_object_move(o, 0, 0);
   evas_object_resize(o, w, h);
   evas_object_layer_set(o, -1000000);
   evas_object_event_callback_add(o, EVAS_CALLBACK_KEY_DOWN, _bg_cb_key_down, ee);
   evas_object_focus_set(o, 1);
   evas_object_show(o);
   o_bg_rect = o;

   o = evas_object_image_add(evas);
   evas_object_image_file_set(o, WP"Default.png", NULL);
   evas_object_image_fill_set(o, 0, 0, w, h);
   evas_object_move(o, 0, 0);
   evas_object_resize(o, w, h);
   evas_object_show(o);
   o_bg = o;
}

void
bg_go(void)
{
   Evas_Object *o;
   int ow, oh;

   evas_output_size_get(evas, &ow, &oh);
   o = edje_object_add(evas);
   edje_object_file_set(o, ED"default.edj", "init/splash");
   edje_object_signal_callback_add(o, "stop", "done", _bg_cb_stop_done, o);
   evas_object_move(o, 0, 0);
   evas_object_resize(o, ow, oh);
   evas_object_show(o);
   o_bg_init = o;
   
   ecore_timer_add(8.0, _bg_cb_timer_init_end, NULL);
}

void
bg_file_set(char *file)
{
   Evas_Coord w, h;
   
   evas_output_viewport_get(evas, NULL, NULL, &w, &h);
   evas_object_image_file_set(o_bg, file, NULL);
   evas_object_image_fill_set(o_bg, 0, 0, w, h);
}

static int
_bg_cb_timer_init_end(void *data)
{
   edje_object_signal_emit(o_bg_init, "stop", "");  
   return 0;
}


/* NB: on ipaq
 * 
 * "Escape"
 * 
 * "F1"   "F2"    "F3"   "F4"
 *            "Up"
 *   "Left" "Return" "Right"
 *           "Down"
 * 
 */
static void
_bg_cb_key_down(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
   Evas_Event_Key_Down *ev;
   
   ev = event_info;
   if (!strcmp(ev->keyname, "Escape")) ecore_main_loop_quit();
   if (!strcmp(ev->keyname, "q")) ecore_main_loop_quit();
   if (!strcmp(ev->keyname, "Q")) ecore_main_loop_quit();
   if (!strcmp(ev->keyname, "Return"))
     {
	menus_start();
     }
}

static void
_bg_cb_stop_done(void *data, Evas_Object *o, const char *emission, const char *source)
{
   evas_object_del(o_bg_init);
   o_bg_init = NULL;
}
